use crossterm::event::{read, Event, Event::Key, KeyCode::{Char, Backspace}, KeyEvent, KeyModifiers};
mod terminal;
use terminal::{Terminal, Position};
mod view;
use view::{View, TyperInfo, ResultInfo};
mod words;
use words::WordList;
use std::io::Error;
use std::time::{SystemTime};

pub struct Typer {
    should_quit: bool,
    should_display_result: bool,
    words_to_write: WordList,
    chars_written: Vec<char>,
    current_num_of_words: u16,
    wpm: u16,
    errors: u16,
    start_timestamp: Option<SystemTime>,
}

impl Typer {
    pub fn default() -> Self {

        Typer{
            should_quit: false,
            should_display_result: false,
            words_to_write: WordList::new(50),
            chars_written: Vec::new() as Vec<char>,
            current_num_of_words: 0,
            wpm: 0,
            errors: 0,
            start_timestamp: None
        }
    }

    pub fn run(&mut self) {
        Terminal::initialize().unwrap();
        let result = self.repl();
        Terminal::terminate().unwrap();
        result.unwrap();
    }

    fn repl(&mut self) -> Result<(), Error> {
        loop {
            self.refresh_screen()?;
            if self.should_quit {
                break;
            }
            let event = read()?;
            self.evaluate_event(&event);
        }
        Ok(())
    }

    fn evaluate_event(&mut self, event: &Event) {
        if let Key(KeyEvent { code, modifiers, .. }) = event {
            match code {
                Char('q') if *modifiers == KeyModifiers::CONTROL => {
                    self.should_quit = true;
                }
                Backspace => {
                    if self.chars_written.pop().unwrap() == ' ' && self.current_num_of_words > 0 {
                        self.current_num_of_words -= 1;
                    }
                }
                _ => {
                    if code.as_char().is_some() {
                        if self.chars_written.len() < self.words_to_write.chars.len() - 2 {
                            // Start the timer for WPM
                            if self.start_timestamp.is_none() {
                                self.start_timestamp = Some(SystemTime::now());
                            }

                            let c = code.as_char().unwrap();

                            // Add an error if the character written is not the correct.
                            if c != *self.words_to_write.chars.get(self.chars_written.len()).unwrap() {
                                self.errors += 1;
                            }

                            // Increase the number of written words on space.
                            if c == ' ' {
                                self.current_num_of_words += 1;
                            }

                            // Update WPM
                            match self.start_timestamp.unwrap().elapsed() {
                                Ok(elapsed) => {
                                    if elapsed.as_secs() != 0 {
                                        self.wpm = (f32::from(self.current_num_of_words) / (elapsed.as_secs() as f32 / 60_f32)) as u16;
                                    }
                                }
                                Err(_e) => {
                                    panic!("Problem with the timer");
                                }
                            }

                            self.chars_written.push(c);
                        } else {
                            self.should_display_result = true;
                        }
                    }
                },
            }
        }
    }

    fn refresh_screen(&mut self) -> Result<(), Error> {
        if self.should_quit {
            Terminal::clear_screen()?;
            Terminal::print("Goodbye.\r\n")?;
        } else if self.should_display_result {
            Terminal::clear_screen()?;
            View::render_result(ResultInfo {
                wpm: self.wpm,
                errors: self.errors,
                accuracy: 100_f32 * (1_f32 - (self.errors as f32 / self.words_to_write.chars.len() as f32)),
            })?;
        } else {
            View::render_typer(TyperInfo {
                words_to_write: self.words_to_write.words.clone(),
                words_written: self.chars_written.clone(),
                wpm: self.wpm,
                errors: self.errors
            })?;
            Terminal::move_cursor_to(Position {x: 0, y: 0})?;
        }
        Terminal::execute()?;
        Ok(())
    }
}
