use crossterm::cursor::{Hide, Show, MoveTo};
use crossterm::queue;
use crossterm::style::{Print, ResetColor, SetForegroundColor, Color};
use crossterm::terminal::{disable_raw_mode, enable_raw_mode, size, Clear, ClearType, EnterAlternateScreen, LeaveAlternateScreen};
use std::io::{stdout, Error, Write};

#[derive(Copy, Clone)]
pub struct Size {
    pub height: u16,
    pub width: u16,
}

#[derive(Copy, Clone)]
pub struct Position {
    pub x: u16,
    pub y: u16,
}

pub struct Terminal;

impl Terminal {
    pub fn terminate() -> Result<(), Error> {
        Self::show_cursor()?;
        Self::execute()?;
        Self::leave_alternate_screen()?;
        disable_raw_mode()?;
        Ok(())
    }

    pub fn initialize() -> Result<(), Error> {
        enable_raw_mode()?;
        Self::enter_alternate_screen()?;
        Self::clear_screen()?;
        Self::move_cursor_to(Position {x: 0, y: 0})?;
        Self::hide_cursor()?;
        Self::execute()?;
        Ok(())
    }
    
    pub fn leave_alternate_screen() -> Result<(), Error> {
        queue!(stdout(), LeaveAlternateScreen)?;
        Ok(())
    }

    pub fn enter_alternate_screen() -> Result<(), Error> {
        queue!(stdout(), EnterAlternateScreen)?;
        Ok(())
    }


    pub fn clear_screen() -> Result<(), Error> {
        queue!(stdout(), Clear(ClearType::All))?;
        Ok(())
    }

    pub fn reset_color() -> Result<(), Error> {
        queue!(stdout(), ResetColor)?;
        Ok(())
    }

    pub fn set_foreground_color(color: Color) -> Result<(), Error> {
        queue!(stdout(), SetForegroundColor(color))?;
        Ok(())
    }

    pub fn move_cursor_to(position: Position) -> Result<(), Error> {
        queue!(stdout(), MoveTo(position.x,position.y))?;
        Ok(())
    }

    pub fn hide_cursor() -> Result<(), Error> {
        queue!(stdout(), Hide)?;
        Ok(())
    }

    pub fn show_cursor() -> Result<(), Error> {
        queue!(stdout(), Show)?;
        Ok(())
    }

    pub fn print(string: &str) -> Result<(), Error> {
        queue!(stdout(), Print(string))?;
        Ok(())
    }

    pub fn size() -> Result<Size, Error> {
        let (width, height) = size()?;
        Ok(Size { height, width })
    }
    
    pub fn execute() -> Result<(), Error> {
        stdout().flush()?;
        Ok(())
    }
}
