use super::terminal::{Size, Position, Terminal};
use crossterm::style::Color;
use std::io::Error;
const NAME: &str = env!("CARGO_PKG_NAME");
const VERSION: &str = env!("CARGO_PKG_VERSION");

pub struct TyperInfo {
    pub words_to_write: Vec<String>,
    pub words_written: Vec<char>,
    pub wpm: u16,
    pub errors: u16,
}

pub struct ResultInfo {
    pub wpm: u16,
    pub errors: u16,
    pub accuracy: f32,
}

pub struct View;

impl View {
    pub fn render_typer(info: TyperInfo) -> Result<(), Error> {
        Self::draw_static_ui()?;
        Self::draw_dynamic_ui(&info)?;
        Ok(())
    }

    pub fn render_result(info: ResultInfo) -> Result<(), Error> {
        Self::draw_footer()?;

        let Size{width, ..} = Terminal::size()?;
        let title = "Bravo ! Voici vos résultats.";

        #[allow(clippy::cast_possible_truncation)]
        Terminal::move_cursor_to(Position {x: (width - title.len() as u16) / 2, y: 5})?;
        Terminal::print(title)?;

        let x_padding = width / 4;

        let min_x = x_padding;
        let min_y = 7;
        Terminal::move_cursor_to(Position {x: min_x, y: min_y})?;
        Terminal::print(&format!("WPM: {}    ", info.wpm))?;

        Terminal::move_cursor_to(Position {x: min_x, y: min_y + 1})?;
        Terminal::print(&format!("Erreurs: {}", info.errors))?;

        Terminal::move_cursor_to(Position {x: min_x, y: min_y + 2})?;
        Terminal::print(&format!("Précision: {:.2}%", info.accuracy))?;
        
        Ok(())
    }

    fn draw_static_ui() -> Result<(), Error> {
        let Size{width, ..} = Terminal::size()?;
        let title = "C'est parti !";

        #[allow(clippy::cast_possible_truncation)]
        Terminal::move_cursor_to(Position {x: (width - title.len() as u16) / 2, y: 5})?;
        Terminal::print(title)?;

        Self::draw_footer()?;
        Ok(())
    }

    fn draw_footer() -> Result<(), Error> {
        let Size{width, height} = Terminal::size()?;
        let version_name = NAME.to_owned() + " - " + VERSION;

        #[allow(clippy::cast_possible_truncation)]
        Terminal::move_cursor_to(Position {x: width - version_name.len() as u16, y: height - 1})?;
        Terminal::print(&version_name)?;
        Terminal::move_cursor_to(Position {x: 0, y: height - 1})?;
        Terminal::print("Appuyez sur CTRL+Q pour quitter")?;
        Ok(())
    }

    fn draw_dynamic_ui(info: &TyperInfo) -> Result<(), Error> {
        let Size{width, ..} = Terminal::size()?;

        let x_padding = width / 4;

        let min_x = x_padding;
        let min_y = 7;
        let max_x = width - x_padding;
        let mut x_offset = 0;
        let mut y_offset = 2;
        let mut char_offset = 0;
        Terminal::move_cursor_to(Position {x: min_x, y: min_y})?;
        Terminal::print(&format!("Erreurs: {}", info.errors))?;

        Terminal::move_cursor_to(Position {x: min_x + 20, y: min_y})?;
        Terminal::print(&format!("WPM: {}    ", info.wpm))?;

        Terminal::move_cursor_to(Position {x: min_x, y: min_y + y_offset})?;

        for i in 0..info.words_to_write.len() {
            let current_word = info.words_to_write.get(i).unwrap();
            #[allow(clippy::cast_possible_truncation)]
            if min_x + x_offset + current_word.len() as u16 > max_x {
                x_offset = 0;
                y_offset += 1;
                Terminal::move_cursor_to(Position {x: min_x + x_offset, y: min_y + y_offset})?;
            }
            for c in current_word.chars() {
                if char_offset < info.words_written.len() {
                    if *info.words_written.get(char_offset).unwrap() == c {
                        Terminal::set_foreground_color(Color::White)?;
                    } else {
                        Terminal::set_foreground_color(Color::Red)?;
                    }
                } else {
                    Terminal::set_foreground_color(Color::DarkGrey)?;
                }
                Terminal::print(&format!("{c}"))?;
                char_offset += 1;
                x_offset += 1;
            }
            Terminal::print(" ")?;
            char_offset += 1;
            x_offset += 1;
        }
        Terminal::reset_color()?;

        Ok(())
    }
}
